/*****************************************************************/
/*
	    C de Cerca 1
	    Introduccin al lenguaje C y su aplicacin
	    para la programacin del Mac
	        
			Rafael Escot - Enric Herrera
			BlauSoft S.L. para MacFormat Espaa - 1996
					
			La descripcin del cdigo y la informacin sobre cmo preparar
			el entorno de programacin y generar estos programas, est contenida
			en el ejemplar de MacFormat que los acompaa.
					

*/
/*****************************************************************/



/****************************************************************	*/
/* DECLARACION DE PROTOTIPOS DE LAS FUNCIONES QUE HEMOS CREADO		*/
/* EN NUESTRO PROGRAMA																						*/ 
/****************************************************************	*/
void		main(void);		/* Esta funcin NO puede tener otro nombre */
void		MF_InitToolbox 		(void);	/* Preparar el Mac */
void 		MF_AbreMiVentana	(void);	/* Abrir una ventana */
void 		MF_EscribeTexto		(void);	/* Poner en ella un texto */
void 		MF_bucle_principal(void);	/* Esperar acciones del usuario */



/****************************************************************	*/
/* CONSTANTES Y VARIABLES GLOBALES									 							*/
/* RELACIONADAS CON LA VENTANA QUE ABRIREMOS											*/ 
/****************************************************************	*/
/* La ventana que abriremos ser creada dinmicamente por medio de
la funcin NewWindow() que nos devolver un puntero a la estructura de
la nueva ventana, Declaramos pues, una variable adecuada... */

WindowPtr		gMiVentana = nil;


																
/*--------------------------------------------------------------------*/
/* A PARTIR DE ESTE PUNTO, APARECE EL CODIGO DE LAS FUNCIONES CREADAS
	 ESPECIFICAMENTE PARA ESTE PROGRAMA. 
/*--------------------------------------------------------------------*/



/****************************************************************	*/
/* FUNCION: main()                                     			      */
/****************************************************************	*/
/*
	Todos los programas en C poseen una funcin que debe llamarse main.
	Cuando un programa se pone en marcha, lo hace SIEMPRE ejecutando
	en primer lugar la funcin main().
	Es el punto de entrada a nuestra aplicacin.
	
	Recibe: 	nada
	Que hace: Llama otras funciones que preparan la aplicacin...
	Salida:  	void (nada)
*/
void	main (void)
{	
	/* Antes de hacer nada, debemos preparar el entorno de nuestro programa.*/
	/* Como sea que el procedimiento a seguir es casi siempre el mismo,			*/
	/* hemos creado una funcin MF_InitToolbox() que realiza la preparacin */
	/* adecuada */ 
	MF_InitToolbox();
	
	/* Ahora empleamos una funcin que se encargar de abrir una ventana */
	MF_AbreMiVentana();
	
	/* Una vez abierta la ventana, podemos escribir algo de texto en ella */
	MF_EscribeTexto();
	
	/* Y ya podemos entrar en el bucle principal, que se encargar de
		 recoger e interpretar las acciones del usuario */	
	MF_bucle_principal();
	
	/* Cuando el bucle principal haya terminado, ser porque el usuario
		 desea acabar el programa.
		 Debemos liberar la memoria ocupada por nuestra ventana. */
	DisposeWindow(gMiVentana);
	
  /* Y ya est, al acabarse la funcin main(), el Sistema cerrar
		 nuestra aplicacin de forma automtica */
	
} /* Fin de main() y del programa */




/****************************************************************	*/
/* MF_bucle_principal		                                         	*/
/****************************************************************	*/
/*
	Recibe: 	nada
	Que hace: Entra en un bucle y espera que se produzca una accin
						En esta demo, simplemente esperamos que el usuario pulse una vez el
						botn del ratn. Cuando lo haga, la funcin acabar y el programa
						volver a la funcin main().
	Salida:  	nada
*/
void MF_bucle_principal(void)
{
	
	while (Button() == 0) ;	/* Esperamos que el usuario pulse el botn */
		
} /* Fin de la funcin MF_bucle_principal() */



/****************************************************************	*/
/* MF_InitToolbox 																								*/
/****************************************************************	*/
/*
	Inicializa la Toolbox Macintosh. El orden de llamada a las diversas
	rutinas es significativo! No lo cambie.

	Recibe: 	nada
	Cambia:  	nada en la aplicacin
	Salida:  	nada
*/
void MF_InitToolbox (void)
{
	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent,0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(0L);
	InitCursor();
								
} /* Fin de MF_InitToolbox() */



/****************************************************************	*/
/* MF_AbreMiVentana		                                           	*/
/****************************************************************	*/
/*
	Abre la ventana (la nica) de nuestro programa. Las funciones empleadas
	y el significado de los parmetros que utilizamos se comentarn en posteriores
	versiones de esta aplicacin.
	
	Recibe: 	nada
	Que hace: Crea una ventana...
	Salida:  	void (nada)
*/
void MF_AbreMiVentana(void)
{
	/* Declaracin e inicializacin de una variable de tipo Rect */					
	Rect	miRectangulo = { 48, 8,108, 300 };	
	
	gMiVentana = NewWindow(	nil,
													&miRectangulo,
													"\pC de Cerca - 1",
													true,
													documentProc,
													(WindowPtr)-1L,
													false,
													nil);
  
  if(gMiVentana == nil) 
  		ExitToShell();	/* Si la creacin de la ventana falla, 
  											posiblemente sea debido a falta de memoria libre
  											Lo ms aconsejable es salir de la aplicacin YA, 
  											empleando ExitToShell()
  										*/
  	
} /* Fin de MF_AbreMiVentana() */



/****************************************************************/
/* MF_EscribeTexto 		    																			*/ 
/****************************************************************/
/*
	Dibuja en nuestra ventana un texto.
		
	Recibe: 	nada
	Cambia:  	Escribe un texto en la ventana
	Salida:   nada
*/
void MF_EscribeTexto()
{

	SetPort(gMiVentana);
	MoveTo(15,20);
	DrawString("\pLa mnima expresin de un programa en C.");
	MoveTo(15,35);
	DrawString("\pHaga clic para salir...");
	 		
} /* Fin de MF_EscribeTexto() */